import 'dart:async';

import 'package:dart_dokr/src/core/file_system.dart';
import 'package:dart_dokr/src/core/stacks.dart';
import 'package:dart_dokr/src/dokr_command.dart';
import 'package:mason_logger/mason_logger.dart';

class CreateCommand extends DokrCommand {
  @override
  String get name => 'create';

  @override
  String get description => 'Create a new stack.';

  @override
  FutureOr<int>? run() async {
    // Get stack names from arguments.
    final stackArgs = argResults!.rest;

    // List all files in stacks directory.
    final stacks = Stacks.listStacks(config);

    for (final stack in stackArgs) {
      if (stacks.contains(stack)) {
        logger.err('Stack already exists: $stack');
        exitCode = ExitCode.usage.code;
        continue;
      }

      final fileName = config.name.template.render({'stack': stack});

      final content = '''
version: "3"

name: "$stack"

services:
  $stack:
    image: nginx
    ports:
     - "8012:80"
    environment:
     - NGINX_HOST=foobar.com
     - NGINX_PORT=80
''';

      final path = FileSystem.createFile(config.stacksDir, fileName, content);

      logger
        ..detail('Created file $path')
        ..success('Created stack $stack, you can now edit $fileName.');
    }

    return super.run() ?? ExitCode.success.code;
  }
}
