import 'dart:async';

import 'package:dart_dokr/src/core/docker.dart';
import 'package:dart_dokr/src/core/stacks.dart';
import 'package:dart_dokr/src/dokr_command.dart';
import 'package:mason_logger/mason_logger.dart';

class ListCommand extends DokrCommand {
  ListCommand() {
    argParser.addFlag(
      'status',
      abbr: 's',
      help: 'Show status of stacks.',
    );
  }

  @override
  String get name => 'list';

  @override
  List<String> get aliases => ['ls'];

  @override
  String get description => 'List all available stacks.';

  @override
  FutureOr<int>? run() async {
    final status = argResults!['status'] as bool;

    // List all files in stacks directory.
    final stacks = Stacks.listStacks(config);

    for (final stack in stacks) {
      if (status) {
        final docker = Docker(
          stackFile: Stacks.findStackFile(config, stack)!,
          stackName: stack,
          config: config,
        );

        final isRunning = await docker.isRunning();

        if (isRunning) {
          logger.info('• $stack ${green.wrap('(running)')}');
        } else {
          logger.info('• $stack ${red.wrap('(stopped)')}');
        }
      } else {
        logger.info('• $stack');
      }
    }

    if (stacks.isEmpty) {
      logger.warn('No stacks found.');
    } else {
      logger.success('Found ${stacks.length} stacks.');
    }

    return super.run() ?? ExitCode.success.code;
  }
}
