import 'package:dart_dokr/src/core/process.dart';
import 'package:dart_dokr/src/models/config.dart';

class DockerException implements Exception {
  DockerException(this.message);

  final String message;

  @override
  String toString() => message;
}

class Docker {
  const Docker({
    required this.stackFile,
    required this.stackName,
    required this.config,
  });

  final Uri stackFile;
  final String stackName;
  final Config config;

  List<String> get envFileArgs {
    final envFile = config.envFiles;
    final args = <String>[];

    for (final file in envFile) {
      args.addAll(['--env-file', file.toFilePath()]);
    }

    return args;
  }

  Future<bool> isRunning() async {
    try {
      final result = await Process.run(
        'docker',
        [
          'compose',
          ...envFileArgs,
          '-f',
          stackFile.toFilePath(),
          'ps',
        ],
        logOutput: false,
      );

      final isRunning = result.stdout.toString().contains('Up');

      return isRunning;
    } catch (_) {
      return false;
    }
  }

  Future<void> start() async {
    final result = await Process.run(
      'docker',
      [
        'compose',
        ...envFileArgs,
        '-f',
        stackFile.toFilePath(),
        'up',
        '-d',
      ],
      logOutput: false,
    );
    if (result.exitCode != 0) {
      throw DockerException(
        'Failed to start stack $stackName: ${result.stderr.toString().trim()}',
      );
    }
  }

  Future<void> stop() async {
    final result = await Process.run(
      'docker',
      [
        'compose',
        ...envFileArgs,
        '-f',
        stackFile.toFilePath(),
        'down',
      ],
      logOutput: false,
    );
    if (result.exitCode != 0) {
      throw DockerException(
        'Failed to stop stack $stackName: ${result.stderr.toString().trim()}',
      );
    }
  }

  Future<void> restart() async {
    await stop();
    await start();
  }
}
