import 'dart:io' as io;

import 'package:dart_dokr/src/core/logging.dart';
import 'package:dart_dokr/src/models/config.dart';
import 'package:mason_logger/mason_logger.dart';
import 'package:path/path.dart';
import 'package:yaml/yaml.dart';

abstract class FileSystem {
  static Logger get logger => Logging.instance;

  static bool checkExists(String path) => io.File(path).existsSync();

  static bool checkUriExists(Uri uri) => io.File.fromUri(uri).existsSync();

  static bool checkDirExists(String path) => io.Directory(path).existsSync();

  static bool checkUriDirExists(Uri uri) =>
      io.Directory.fromUri(uri).existsSync();

  static Config parseConfig(String path) {
    final file = io.File(path);
    final contents = file.readAsStringSync();
    final yaml = loadYaml(contents);
    return Config.fromJson(yaml as Map);
  }

  static List<Uri> listFiles(Uri path) {
    final dir = io.Directory.fromUri(path);
    final files = dir.listSync();
    return files.map((file) => file.uri).toList();
  }

  static Uri createFile(Uri path, String name, [String? content]) {
    final file = io.File(join(path.path, name))..createSync();
    if (content != null) {
      file.writeAsStringSync(content);
    }
    return file.uri;
  }
}
