import 'package:dart_dokr/src/core/file_system.dart';
import 'package:dart_dokr/src/core/logging.dart';
import 'package:dart_dokr/src/models/config.dart';
import 'package:mason_logger/mason_logger.dart';
import 'package:path/path.dart';

abstract class Stacks {
  static Logger get logger => Logging.instance;

  static List<Uri>? _stackFiles;

  static List<Uri> listStackFiles(Config config) {
    if (_stackFiles != null) {
      return _stackFiles!;
    }

    // List all files in stacks directory.
    final stacks = FileSystem.listFiles(config.stacksDir)
        .where((file) => config.name.regex.hasMatch(basename(file.path)))
        .toList();

    return _stackFiles = stacks;
  }

  static List<String> listStacks(Config config) {
    // List all files in stacks directory.
    final stacks = listStackFiles(config).map((file) {
      logger.detail('Found stack: ${basename(file.path)}');

      return config.name.regex
          .firstMatch(basename(file.path))!
          .namedGroup('Stack')!;
    }).toList();

    return stacks;
  }

  static Uri? findStackFile(
    Config config,
    String stack, {
    bool refresh = false,
  }) {
    if (refresh) {
      _stackFiles = null;
    }
    // List all files in stacks directory.
    for (final file in listStackFiles(config)) {
      final match = config.name.regex.firstMatch(basename(file.path))!;
      if (match.namedGroup('Stack') == stack) {
        return file;
      }
    }

    return null;
  }
}
