import 'dart:async';
import 'dart:io';

import 'package:args/args.dart';
import 'package:args/command_runner.dart';
import 'package:dart_dokr/src/core/constants.dart';
import 'package:dart_dokr/src/core/file_system.dart';
import 'package:dart_dokr/src/core/logging.dart';
import 'package:dart_dokr/src/models/config.dart';
import 'package:mason_logger/mason_logger.dart';
import 'package:meta/meta.dart';

abstract class DokrCommand extends Command<int> {
  DokrCommand();

  /// [Logger] instance used to wrap stdout.
  Logger get logger => Logging.instance;

  /// [ArgResults] used for testing purposes only.
  @visibleForTesting
  ArgResults? testArgResults;

  /// [ArgResults] for the current command.
  ArgResults get results => testArgResults ?? argResults!;

  /// Return the current working directory.
  Directory get cwd => Directory.current;

  /// The code to exit the process with.
  int exitCode = ExitCode.success.code;

  Config? _config;

  /// The [Config] instance for the current command.
  Config get config {
    if (_config != null) {
      return _config!;
    }

    final cfg = globalResults?['config'] as String?;

    try {
      final configParsed =
          FileSystem.parseConfig(cfg ?? kDefaultConfigFileName);
      logger.detail('Config parsed successfully.');
      return _config = configParsed;
    } catch (e) {
      logger.detail(e.toString());
      usageException('Config file `$cfg` not found.');
    }
  }

  @override
  FutureOr<int>? run() => exitCode;
}
